#include <QtCore>
#include <QPainter>
#include <QEvent>
#include <QGraphicsSceneEvent>
#include "graphicsstroke.h"

GraphicsStroke::GraphicsStroke(QGraphicsItem *parent) :
    QGraphicsItem(parent)
{
    m_pen.setColor(Qt::red);
    m_pen.setStyle(Qt::SolidLine);
    m_pen.setWidth(3);
}

void GraphicsStroke::paint ( QPainter * painter, const QStyleOptionGraphicsItem * option, QWidget * widget )  {
    Q_UNUSED(option);
    Q_UNUSED(widget);

    if (m_polygon.size() > 0){
        painter->save();

        painter->setPen(m_pen);
        painter->drawLines(m_polygon);

        painter->restore();
    }
}

QRectF GraphicsStroke::boundingRect() const {
    return m_boundingRect;
}

QPainterPath GraphicsStroke::shape () const {
    QPainterPath path;
    path.addPolygon(m_polygon);
    return path;
}

QString GraphicsStroke::tagName(){
    return "stroke";
}

QDomElement GraphicsStroke::createDomElement(QDomDocument &doc){
    QDomElement element = doc.createElement(tagName());

    QStringList list;
    QVector<QPointF> pts = points();
    foreach (QPointF pt , pts) {
        QString str = QString("%1,%2").arg(pt.x()).arg(pt.y());
        list << str;
    }

    element.setAttribute("points",list.join(" "));
    element.setAttribute("color",m_pen.color().name());
    element.setAttribute("width",m_pen.width());
    return element;
}

bool GraphicsStroke::setFromDomElement(QDomElement &elem){
    if (elem.tagName() != tagName())
        return false;

    clear();
    QString color = elem.attribute("color","#000000");
    QColor c(color);
    m_pen.setColor(c);

    int width = elem.attribute("width","1").toInt();
    m_pen.setWidth(width);

    QString val = elem.attribute("points","");
    QStringList list = val.split(" ");

//    QVector<QPointF> pts;
    QPointF last;

    for (int i = 0 ; i < list.size() ; i++) {
        QStringList xy = list.at(i).split(",");

        if (xy.size() == 2) {
            qreal x = xy.at(0).toFloat();
            qreal y = xy.at(1).toFloat();
            QPointF pt(x,y);

            if (i != 0) {
                append(last,false);
                append(pt,false);
            } else {
                append(pt,false);
                append(pt,false);
            }
            last = pt;
        }

    }

    updateBoundingRect();
    prepareGeometryChange();
    return true;
}

QPolygonF GraphicsStroke::polygon(){
    return m_polygon;
}

void GraphicsStroke::setPolygon(QPolygonF val){
    m_polygon = val;
    updateBoundingRect();
    prepareGeometryChange();
}

QVector<QPointF> GraphicsStroke::points(){
    QVector <QPointF> res;
    int n = m_polygon.size();
    if (n > 0){
        QPointF prev = m_polygon.at(0);
        res << prev;

        for (int i = 1 ; i< n ;i++){
            QPointF pt = m_polygon.at(i);
//            qDebug() << prev << pt ;
            if (pt!= prev) {
//            if ( !(pt.x() == prev.x() && pt.y() == prev.y())){
                res << pt;
                prev = pt;
            }
        }
    }

    return res;
}


void GraphicsStroke::append(const QPointF pt,bool update){
    m_polygon.append(pt);

    if (update){
        updateBoundingRect();
        prepareGeometryChange();
    }
}

void GraphicsStroke::clear() {
    m_polygon.clear();
    updateBoundingRect();
    prepareGeometryChange();
}

void GraphicsStroke::setPen(const QPen& val){
    m_pen = val;
}

QPen GraphicsStroke::pen(){
    return m_pen;
}

void GraphicsStroke::updateBoundingRect(){
    m_boundingRect = m_polygon.boundingRect();
}
